@@ -24,7 +24,9 @@
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#if __has_include("config.hpp")
 #include "config.hpp"
+#endif
 
 #include <exception>
 #include <iostream>
@@ -34,103 +36,12 @@
 #include <boost/format.hpp>
 #include <boost/date_time/gregorian/formatters.hpp>
 
-#include <cif++/Config.hpp>
-#include <cif++/Structure.hpp>
-#include <cif++/Secondary.hpp>
 #include <cif++/CifUtils.hpp>
 #include <cif++/Cif2PDB.hpp>
 
-#include <boost/program_options.hpp>
-#include <boost/iostreams/filter/gzip.hpp>
-#include <boost/iostreams/filter/bzip2.hpp>
-#include <boost/iostreams/filtering_stream.hpp>
-
-namespace fs = std::filesystem;
-namespace io = boost::iostreams;
-namespace po = boost::program_options;
-
-// --------------------------------------------------------------------
-
-// recursively print exception whats:
-void print_what (const std::exception& e)
-{
-	std::cerr << e.what() << std::endl;
-	try
-	{
-		std::rethrow_if_nested(e);
-	}
-	catch (const std::exception& nested)
-	{
-		std::cerr << " >> ";
-		print_what(nested);
-	}
-}
-
-// --------------------------------------------------------------------
-
-namespace {
-	std::string gVersionNr, gVersionDate, VERSION_STRING;
-}
-
-void load_version_info()
-{
-	const std::regex
-		rxVersionNr(R"(build-(\d+)-g[0-9a-f]{7}(-dirty)?)"),
-		rxVersionDate(R"(Date: +(\d{4}-\d{2}-\d{2}).*)"),
-		rxVersionNr2(R"(dssp-version: (\d+(?:\.\d+)+))");
-
+#include "dssp.hpp"
 #include "revision.hpp"
 
-	struct membuf : public std::streambuf
-	{
-		membuf(char* data, size_t length)       { this->setg(data, data, data + length); }
-	} buffer(const_cast<char*>(kRevision), sizeof(kRevision));
-
-	std::istream is(&buffer);
-
-	std::string line;
-
-	while (getline(is, line))
-	{
-		std::smatch m;
-
-		if (std::regex_match(line, m, rxVersionNr))
-		{
-			gVersionNr = m[1];
-			if (m[2].matched)
-				gVersionNr += '*';
-			continue;
-		}
-
-		if (std::regex_match(line, m, rxVersionDate))
-		{
-			gVersionDate = m[1];
-			continue;
-		}
-
-		// always the first, replace with more specific if followed by the other info
-		if (std::regex_match(line, m, rxVersionNr2))
-		{
-			gVersionNr = m[1];
-			continue;
-		}
-	}
-
-	if (not VERSION_STRING.empty())
-		VERSION_STRING += "\n";
-	VERSION_STRING += gVersionNr + " " + gVersionDate;
-}
-
-std::string get_version_nr()
-{
-	return gVersionNr/* + '/' + cif::get_version_nr()*/;
-}
-
-std::string get_version_date()
-{
-	return gVersionDate;
-}
-
 // --------------------------------------------------------------------
 
 std::string ResidueToDSSPLine(const mmcif::DSSP::ResidueInfo& info)
@@ -182,7 +93,7 @@ std::string ResidueToDSSPLine(const mmcif::DSSP::ResidueInfo& info)
 			case mmcif::Helix::Start:		helix[static_cast<int>(helixType)] = '>'; break;
 			case mmcif::Helix::End:			helix[static_cast<int>(helixType)] = '<'; break;
 			case mmcif::Helix::StartAndEnd:	helix[static_cast<int>(helixType)] = 'X'; break;
-			case mmcif::Helix::Middle:		helix[static_cast<int>(helixType)] = (helixType == mmcif::HelixType::rh_pp ? 'P' : ('3' + static_cast<int>(helixType))); break;
+			case mmcif::Helix::Middle:		helix[static_cast<int>(helixType)] = (helixType == mmcif::HelixType::rh_pp ? 'P' : static_cast<char>('3' + static_cast<int>(helixType))); break;
 		}
 	}
 
@@ -243,7 +154,7 @@ std::string ResidueToDSSPLine(const mmcif::DSSP::ResidueInfo& info)
 
 void writeDSSP(const mmcif::Structure& structure, const mmcif::DSSP& dssp, std::ostream& os)
 {
-	const std::string kFirstLine("==== Secondary Structure Definition by the program DSSP, NKI version 3.0                           ==== ");
+	const std::string kFirstLine("==== Secondary Structure Definition by the program DSSP, NKI version 4.0                           ==== ");
 	boost::format kHeaderLine("%1% %|127t|%2%");
 
 	using namespace boost::gregorian;
@@ -441,173 +352,7 @@ void annotateDSSP(mmcif::Structure& structure, const mmcif::DSSP& dssp, bool wri
 		}
 	}
 
-	db.add_software("dssp", "other", get_version_nr(), get_version_date());
+	db.add_software("dssp", "other", kVersionNumber, kBuildDate);
 
 	db.write(os);
 }
-
-// --------------------------------------------------------------------
-
-int d_main(int argc, const char* argv[])
-{
-	using namespace std::literals;
-
-	po::options_description visible_options(argv[0] + " [options] input-file [output-file]"s);
-	visible_options.add_options()
-		("dict",				po::value<std::vector<std::string>>(),
-															"Dictionary file containing restraints for residues in this specific target, can be specified multiple times.")
-		("output-format",		po::value<std::string>(),	"Output format, can be either 'dssp' for classic DSSP or 'mmcif' for annotated mmCIF. The default is chosen based on the extension of the output file, if any.")
-		("min-pp-stretch",		po::value<short>(),			"Minimal number of residues having PSI/PHI in range for a PP helix, default is 3")
-		("write-other",										"If set, write the type OTHER for loops, default is to leave this out")
-
-		("help,h",											"Display help message")
-		("version",											"Print version")
-		("verbose,v",										"verbose output")
-		;
-	
-	po::options_description hidden_options("hidden options");
-	hidden_options.add_options()
-		("xyzin",				po::value<std::string>(),	"coordinates file")
-		("output",              po::value<std::string>(),	"Output to this file")
-		("debug,d",				po::value<int>(),			"Debug level (for even more verbose output)")
-		;
-
-	po::options_description cmdline_options;
-	cmdline_options.add(visible_options).add(hidden_options);
-
-	po::positional_options_description p;
-	p.add("xyzin", 1);
-	p.add("output", 1);
-	
-	po::variables_map vm;
-	po::store(po::command_line_parser(argc, argv).options(cmdline_options).positional(p).run(), vm);
-
-	po::notify(vm);
-
-	// --------------------------------------------------------------------
-
-	if (vm.count("version"))
-	{
-		std::cout << argv[0] << " version " << VERSION_STRING << std::endl;
-		exit(0);
-	}
-
-	if (vm.count("help"))
-	{
-		std::cerr << visible_options << std::endl;
-		exit(0);
-	}
-	
-	if (vm.count("xyzin") == 0)
-	{
-		std::cerr << "Input file not specified" << std::endl;
-		exit(1);
-	}
-
-	if (vm.count("output-format") and vm["output-format"].as<std::string>() != "dssp" and vm["output-format"].as<std::string>() != "mmcif")
-	{
-		std::cerr << "Output format should be one of 'dssp' or 'mmcif'" << std::endl;
-		exit(1);
-	}
-
-	cif::VERBOSE = vm.count("verbose") != 0;
-	if (vm.count("debug"))
-		cif::VERBOSE = vm["debug"].as<int>();
-
-	// --------------------------------------------------------------------
-	
-	if (vm.count("dict"))
-	{
-		for (auto dict: vm["dict"].as<std::vector<std::string>>())
-			mmcif::CompoundFactory::instance().pushDictionary(dict);
-	}
-
-	mmcif::File f(vm["xyzin"].as<std::string>());
-	mmcif::Structure structure(f, 1, mmcif::StructureOpenOptions::SkipHydrogen);
-
-	// --------------------------------------------------------------------
-
-	short pp_stretch = 3;
-	if (vm.count("min-pp-stretch"))
-		pp_stretch = vm["min-pp-stretch"].as<short>();
-
-	bool writeOther = vm.count("write-other");
-
-	std::string fmt;
-	if (vm.count("output-format"))
-		fmt = vm["output-format"].as<std::string>();
-
-	if (fmt.empty() and vm.count("output"))
-	{
-		fs::path p = vm["output"].as<std::string>();
-	
-		if (p.extension() == ".gz")
-			p = p.stem();
-		else if (p.extension() == ".bz2")
-			p = p.stem();
-
-		if (p.extension() == ".dssp")
-			fmt = "dssp";
-		else
-			fmt = "cif";
-	}
-
-
-	mmcif::DSSP dssp(structure, pp_stretch, fmt == "dssp");
-
-	if (vm.count("output"))
-	{
-		fs::path p = vm["output"].as<std::string>();
-		std::ofstream of(p, std::ios_base::out | std::ios_base::binary);
-
-		if (not of.is_open())
-		{
-			std::cerr << "Could not open output file" << std::endl;
-			exit(1);
-		}
-
-		io::filtering_stream<io::output> out;
-		
-		if (p.extension() == ".gz")
-			out.push(io::gzip_compressor());
-		else if (p.extension() == ".bz2")
-			out.push(io::bzip2_compressor());
-
-		out.push(of);
-
-		if (fmt == "dssp")
-			writeDSSP(structure, dssp, out);
-		else
-			annotateDSSP(structure, dssp, writeOther, out);
-	}
-	else
-	{
-		if (fmt == "dssp")
-			writeDSSP(structure, dssp, std::cout);
-		else
-			annotateDSSP(structure, dssp, writeOther, std::cout);
-	}
-	
-	return 0;
-}
-
-// --------------------------------------------------------------------
-
-int main(int argc, const char* argv[])
-{
-	int result = 0;
-
-	try
-	{
-		load_version_info();
-
-		result = d_main(argc, argv);
-	}
-	catch (const std::exception& ex)
-	{
-		print_what(ex);
-		exit(1);
-	}
-
-	return result;
-}
